
// Operations on collapsed ranges.

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include "globals.h"
#include "Procs.h"
#include "Range.h"


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CollapsedRangeRep': collapsed template range.                //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* CollapsedRangeRep :: subrng(const int _extent,
                                      const int _base, const int _stride) {
  return new CollapsedSubRangeRep(_extent, _base, _stride, this, extent) ;
}

RangeRep* CollapsedRangeRep :: ker() const {
  return new DimRangeRep() ;
}

Format CollapsedRangeRep :: format() const {
  return DIST_COLLAPSED ;
}

int CollapsedRangeRep :: ker_glb(const int glb, int* sub) const {
  *sub = glb ;

  return 0 ;
}

void CollapsedRangeRep :: location(Location* loc, const int glb) const {

  loc->subs[0] = glb ;
  loc->crd     = 0 ;
}

RangeRep* CollapsedRangeRep :: shell() const {
  return (RangeRep*) this ;  // cast away `const'
}

int CollapsedRangeRep :: block(const int, int* sub, int* count) const {
  *sub   = 0 ;
  *count = extent ;

  return 0 ;
}

int CollapsedRangeRep :: block_step(int* sub_stp) const {
  *sub_stp = 1 ;

  return 1 ;
}

RangeRep* CollapsedRangeRep :: subker(int* ker_bas, int* ker_str) const {
  *ker_bas = 0 ;
  *ker_str = 1 ;
  return new DimRangeRep() ;
}

int CollapsedRangeRep :: width() const {return extent ; }

void CollapsedRangeRep :: show() {
  AdlibPrintf("CollapsedRangeRep :: show: "
              "size = %d ker = %p lev = %d dim = %p\n",
              extent, kernel, level, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(CollapsedRangeRep)


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `CollapsedSubRangeRep': subrange of a collapsed range.        //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* CollapsedSubRangeRep :: subrng(const int _extent,
                                         const int _base, const int _stride) {
  return new CollapsedSubRangeRep(_extent,
                                  base + stride * _base, stride * _stride,
                                  parent, parSize) ;
}

RangeRep* CollapsedSubRangeRep :: tem() {return parent ; }

int CollapsedSubRangeRep :: bas() const {return base ; }

int CollapsedSubRangeRep :: str() const {return stride ; }

RangeRep* CollapsedSubRangeRep :: ker() const {
  return new DimRangeRep() ;
}

Format CollapsedSubRangeRep :: format() const {
  return DIST_COLLAPSED ;
}

int CollapsedSubRangeRep :: ker_glb(const int glb, int* sub) const {
  *sub = base + stride * glb ;

  return 0 ;
}

void CollapsedSubRangeRep :: location(Location* loc, const int glb) const {

  loc->subs[0] = base + stride * glb ;
  loc->crd     = 0 ;
}

RangeRep* CollapsedSubRangeRep :: shell() const {
  return (RangeRep*) this ;  // cast away `const'
}

int CollapsedSubRangeRep :: block(const int,
                                  int* sub, int* count) const {
  *sub   = base ;
  *count = extent ;

  return 0 ;
}

int CollapsedSubRangeRep :: block_step(int* sub_stp) const {
  *sub_stp = stride ;

  return 1 ;
}

RangeRep* CollapsedSubRangeRep :: subker(int* ker_bas, int* ker_str) const {
  *ker_bas = 0 ;
  *ker_str = 1 ;
  return new DimRangeRep() ;
}

int CollapsedSubRangeRep :: width() const {return parSize ; }

int CollapsedSubRangeRep :: glb(const int sub, const int) const {
  return (sub - base) / stride ;
}

void CollapsedSubRangeRep :: show() {
  AdlibPrintf("CollapsedSubRangeRep :: show: "
              "size = %d ker = %p lev = %d dim = %p\n",
              extent, kernel, level, (DimensionRep*) dimension) ; 
}

MEMORY_POOL_DEFN(CollapsedSubRangeRep)


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// classes `ExtendedCollapsedRangeRep',                                //
//         `ExtendedCollapsedSubRangeRep':                             //
//     Collapsed range of array with ghost areas.                      //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

RangeRep* ExtendedCollapsedRangeRep :: subrng(const int _extent,
                                              const int _base,
                                              const int _stride) {
  return new ExtendedCollapsedSubRangeRep(_extent, _base, _stride,
                                          this, parSize, wlo, whi) ;
}

int ExtendedCollapsedRangeRep :: disp(const int sub) const {
  return sub + wlo ;
}

int ExtendedCollapsedRangeRep :: width() const {
  return wlo + parSize + whi ;
}

RangeRep* ExtendedCollapsedSubRangeRep :: subrng(const int _extent,
                                                 const int _base,
                                                 const int _stride) {
  return new ExtendedCollapsedSubRangeRep(_extent,
                                          base + stride * _base,
                                          stride * _stride,
                                          parent, parSize, wlo, whi) ;
}

int ExtendedCollapsedSubRangeRep :: disp(const int sub) const {
  return sub + wlo ;
}

int ExtendedCollapsedSubRangeRep :: width() const {
  return wlo + parSize + whi ;
}


MEMORY_POOL_DEFN(ExtendedCollapsedRangeRep)

MEMORY_POOL_DEFN(ExtendedCollapsedSubRangeRep)


// -------------------------------------------------------------
// Things to do
//

